/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/



package org.thermostat.qa.framework;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;



import org.thermostat.qa.annotations.SinceVersion;
import org.thermostat.qa.annotations.StorageType;
import org.thermostat.qa.annotations.StorageTypes;
import org.thermostat.qa.annotations.TestType;
import org.thermostat.qa.annotations.TestTypes;
import org.thermostat.qa.annotations.TillVersion;
import org.thermostat.qa.annotations.ValidVersions;
import org.thermostat.qa.common.Configuration;



/**
 * This is test engine which contains basic methods which are used to run the
 * particular test suites. This class also contains various assertion-like
 * methods (at this moment, only a few assertion types are needed for testing).
 *
 * @author Pavel Tisnovsky
 */
public abstract class ThermostatTest extends ThermostatUtilities
{
    private static final int LT = -1;

    private static final int GT = 1;

    private static final int EQ = 0;

    /**
     * Number of tests that passed.
     */
    private int passed = 0;

    /**
     * Number of tests that failed.
     */
    private int failed = 0;

    /**
     * Number of tests that failed due to other error.
     */
    private int error = 0;

    /**
     * Number of tests that have been ignored.
     */
    private int ignored = 0;
    
    /**
     * Is the current test a gui test?
     */
    private boolean guiTest = false;
    
    /**
     * Do we want to clean all thermostat processes after each gui test?
     * (May do problems on rhel)
     */
    private boolean performAfterGuiCleaning = false;
    
    /**
     * Does the current test use web storage?
     */
    private boolean webStorage = false;

    /**
     * Set up the test suite. To be implemented by specific test cases.
     */
    protected abstract void setUp();

    /**
     * Tear down the test suite. To be implemented by specific test cases.
     */
    protected abstract void tearDown();

    /**
     * Run all tests found in the current test suite.
     * 
     * @param args
     *            arguments which are passed through command line
     */
    protected void doTests(String[] args) {
        long t1 = System.currentTimeMillis();

        this.configuration = new Configuration(args);

        //determine if the class contains gui tests
        TestType testType = getClass().getAnnotation(TestType.class);
        if(testType != null)
        {
            guiTest = (testType.value() == TestTypes.GUI_TEST);
        }
        StorageType storageType = getClass().getAnnotation(StorageType.class);
        if(storageType != null)
        {
            webStorage = (storageType.value() == StorageTypes.WEBTOMCAT_STORAGE);
        }
        
        performAfterGuiCleaning = this.configuration.getPerformCleaningAfterGuiTests();
        // setup test
        setUp();
        try
        {
            if( guiTest )
            {
                cleanDBBeforeShowingInGui();
            }
            runAllTests();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            try
            {
                failedTestCleanUp();
            }
            catch (IOException e1)
            {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
        }
        finally {
            // tear down test
            tearDown();
            
            // check test duration
            long t2 = System.currentTimeMillis();
            // print all results
            printResults(t1, t2, this.passed, this.failed, this.error, this.ignored);

            //cleanup after an attempt to run all tests
            //there could be hanging gui or this testsuite class
            if(guiTest && performAfterGuiCleaning)
            {
                try
                {
                    afterGuiTestsuiteCleanUp();
                }
                catch (IOException e)
                {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
    }

    private void cleanDBBeforeShowingInGui() {
        // TODO Auto-generated method stub
        startStorage();
        try
        {
            runHelperBashScript("delete_longlist_from_db.sh");
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        try
        {
            stopStorage();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
    }

    private void failedTestCleanUp() throws IOException {
        runHelperBashScript("tests-cleanAllThermostats.sh"); //agent, gui
        if(webStorage)
        {
            stopWebStorage();
        }else{
            stopStorage();
        }
    }

    private void afterGuiTestsuiteCleanUp() throws IOException {
        List<String> scriptContent = new ArrayList<String>();
        String s = "kill $(jps | grep "+this.getClass().getSimpleName()+" | cut -f1 -d' ')";
        scriptContent.add(s);
        runBashScriptWithContent("guitests-cleanOneTest.sh", scriptContent);
    }
    
    
    /**
     * Run all tests in the test suite.
     * 
     * @throws Exception
     *             this exception can be thrown by any test.
     */
    private void runAllTests() throws Exception
    {
        // first we get all methods declared in a class
        Method[] methods = getClass().getDeclaredMethods();
        // then we check, if this method is a proper test method
        for (Method method : methods)
        {
            tryToInvokeTestMethod(method);
        }
    }

    private boolean validTest(Method method) {
        SinceVersion sinceVersion = method.getAnnotation(SinceVersion.class);
        TillVersion tillVersion = method.getAnnotation(TillVersion.class);
        ValidVersions validVersions = method.getAnnotation(ValidVersions.class);

        // check if test has any annotation before other checks
        if (tillVersion == null && sinceVersion == null && validVersions == null) {
            return true;
        }

        String actualVersion = this.configuration.getThermostatVersion();

        boolean sinceVersionOk = processSinceVersion(sinceVersion, actualVersion);
        boolean tillVersionOk = processTillVersion(tillVersion, actualVersion);
        boolean validVersionsOk = processValidVersions(validVersions, actualVersion);

        return sinceVersionOk && tillVersionOk && validVersionsOk;
    }

    private boolean processSinceVersion(SinceVersion sinceVersion, String actualVersion)
    {
        if (sinceVersion != null)
        {
            String since = sinceVersion.value();
            int comparison = compareVersions(since, actualVersion);
            return comparison == LT || comparison == EQ;
        }
        // null
        return true;
    }

    private boolean processTillVersion(TillVersion tillVersion, String actualVersion)
    {
        if (tillVersion != null)
        {
            String till = tillVersion.value();
            int comparison = compareVersions(till, actualVersion);
            return comparison == GT || comparison == EQ;
        }
        // null
        return true;
    }

    private boolean processValidVersions(ValidVersions validVersions, String actualVersion) {
        if (validVersions != null)
        {
            String[] versions = validVersions.value();
            for (String version : versions)
            {
                if (compareVersions(version, actualVersion) == EQ)
                {
                    return true;
                }
            }
            return false;
        }
        // null
        return true;
    }

    private int compareVersions(String testVersion, String actualVersion) {
        int[] testMajorMinorRelease = parseMajorMinorRelease(testVersion);
        int[] actualMajorMinorRelease = parseMajorMinorRelease(actualVersion);
        return performComparison(testMajorMinorRelease, actualMajorMinorRelease);
    }

    /**
     * @param testMajorMinorRelease
     * @param actualMajorMinorRelease
     * @return
     */
    private int performComparison(int[] testMajorMinorRelease, int[] actualMajorMinorRelease) {
        for (int i = 0; i < 3; i++)
        {
            int test = testMajorMinorRelease[i];
            int actual = actualMajorMinorRelease[i];
            if (test < actual) return LT;
            if (test > actual) return GT;
        }
        return EQ;
    }

    /**
     * Parse a string in a format "1.2.3" into an int array containing values [1,2,3].
     * 
     * @param versionString
     * @return
     */
    private int[] parseMajorMinorRelease(String versionString) {
        String[] str = versionString.split("\\.");
        return new int[] {Integer.parseInt(str[0]), Integer.parseInt(str[1]), Integer.parseInt(str[2])};
    }

    /**
     * Call a method if its name begins with "test" prefix. Such methods should
     * have public or protected access declaration.
     * 
     * @param method
     *            method which is called
     * @throws Exception
     *            this exception can be thrown by any test.
     */
    private void tryToInvokeTestMethod(Method method) throws Exception
    {
        String methodName = method.getName();
        // all test methods should have a prefix "test"
        if (methodName.startsWith("test"))
        {
            if (!validTest(method)) {
                testIgnored(methodName);
                return;
            }
            try
            {
                method.invoke(this);
                testPassed(method, methodName);
            }
            catch (IllegalAccessException e)
            {
                testError(methodName, e);
            }
            catch (IllegalArgumentException e)
            {
                testError(methodName, e);
            }
            catch (InvocationTargetException e)
            {
                if (e.getTargetException() instanceof AssertionError)
                {
                    testFailed(methodName, e);
                }
                else
                {
                    testError(methodName, e);
                }
            }
            catch (AssertionError e)
            {
                testFailed(methodName, e);
            }
            catch (Exception e)
            {
                testFailed(methodName, e);
            }
        }
    }

    /**
     * @param method
     * @param methodName
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    private void testPassed(Method method, String methodName) throws IllegalAccessException, InvocationTargetException
    {
        printTestPassed(methodName);
        this.passed++;
    }

    /**
     * @param methodName
     * @param e
     * @throws IOException 
     */
    private void testError(String methodName, Exception e) throws IOException
    {
        printTestError(methodName, e);
        this.error++;
        e.printStackTrace();
        failedTestCleanUp();
    }

    /**
     * @param methodName
     * @param e
     * @throws IOException 
     */
    private void testFailed(String methodName, Throwable e) throws IOException
    {
        printTestFailed(methodName, e);
        this.failed++;
        e.printStackTrace();
        failedTestCleanUp();
    }

    /**
     * @param methodName
     * @param e
     */
    private void testIgnored(String methodName)
    {
        printTestIgnored(methodName);
        this.ignored++;
    }

    /**
     * Print information about test which passed.
     * 
     * @param methodName
     *            name of method implementing the test.
     */
    private void printTestPassed(String methodName) {
        log("PASSED", '.', methodName);
    }

    /**
     * Print information about test which failed.
     * 
     * @param methodName
     *            name of method implementing the test.
     */
    private void printTestFailed(String methodName, Throwable e) {
        log("FAILED", '.', methodName + ": FAILED " + e.getMessage());
    }

    /**
     * Print information about test which have been ignored.
     * 
     * @param methodName
     *            name of method implementing the test.
     */
    private void printTestIgnored(String methodName) {
        log("IGNORED", '.', methodName);
    }

    /**
     * Print information about test which failed due to other error.
     * 
     * @param methodName
     *            name of method implementing the test.
     */
    private void printTestError(String methodName, Exception e) {
        log("ERROR", '.', methodName + ": ERROR " + e.getMessage());
    }

    /**
     * Print summary about all tests in one test suite.
     * 
     * @param t1
     *            time when the test suite was started
     * @param t2
     *            time when the test suite finishes
     * @param passedTests
     *            number of passed tests
     * @param failedTests
     *            number of failed tests
     * @param ignoredTests
     *            number of tests which have been ignored
     */
    private void printResults(long t1, long t2, int passedTests,
            int failedTests, int errorTests, int ignoredTests)
    {
        log("SUMMARY", ' ', "   total: " + (passedTests + failedTests + errorTests + ignoredTests) +
            "    passed: " + passedTests +
            "    failed: " + failedTests +
            "    error: " + errorTests +
            "    ignored: " + ignoredTests +
            "    duration: " + (t2-t1) + " ms");
    }

    /**
     * Comparison of two strings/lines.
     *
     * @param line1
     * @param line2
     * @throws AssertionError
     */
    protected void compareTwoLines(String line1, String line2) throws AssertionError
    {
        Assert.assertEquals(line1, line2, line1 + " != " + line2);
    }
  
    /**
     * Comparison of contents of list of strings against array of strings.
     *
     * @param list
     * @param wantedArr
     */
    protected void compareListAndArrayOfStrings(List<String> list, String[] wantedArr)
    {
        int i = 0;
        for(String wanted : wantedArr)
        {
            compareTwoLines(list.get(i), wanted);
            i++;
        }
    }
  
    /**
     * Method checkPresenceOfPatterns returns true iff all the patterns are
     * present in text.
     * 
     * @param text
     * @param patterns
     * @return
     */
    protected boolean checkPresenceOfPatterns(String text, String[] patterns) {
        boolean result = true;
        for(String p : patterns)
        {
            if(!text.contains(p))
            {
                result = false;
            }
        }
        return result;
    }
    
    /**
     * Method checkPresenceOfPatternInTexts
     * returns true if a pattern is present in at least one of the texts
     * in the list.
     * 
     * @param texts
     * @param pattern
     * @return
     */
    protected boolean checkPresenceOfPatternInTexts(List<String> texts, String pattern){
        boolean result = false;
        for(String t : texts)
        {
            if(t.contains(pattern))
            {
                result = true;
            }
        }
        return result;
    }
    
    /**
     * Method checkPresenceOfPatterns
     * returns true if all the patterns can be found somewhere in the texts
     * from given list.
     * 
     * @param texts
     * @param patterns
     * @return
     */
    protected boolean checkPresenceOfPatterns(List<String> texts, String[] patterns){
        boolean result = true;
        for(String p : patterns)
        {
            result = result && checkPresenceOfPatternInTexts(texts,p);
        }
        return result;
    }

}
