/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.reporter;



import java.io.File;
import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;



/**
 * Instances of this class are used to store all command line parameters which
 * can be passed to ThermostatQA report tool.
 * 
 * @author Pavel Tisnovsky <ptisnovs@redhat.com>
 */
public class CommandLineParameters
{
    /**
     * Prefix for a command line parameter used to specify list of tests for
     * which report is to be generated.
     * @see #tests
     */
    private static final String PARAM_NAME_TEST_PREFIX = "-tests=";

    /**
     * Prefix for a command line parameter used to specify date when tests were
     * run.
     * @see #date
     */
    private static final String PARAM_NAME_DATE_PREFIX = "-date=";

    /**
     * Prefix for a command line parameter used to specify directory into which
     * generated report is to be stored.
     * @see #reportDir
     */
    private static final String PARAM_NAME_REPORT_DIR_PREFIX = "-report-dir=";

    /**
     * Prefix for a command line parameter used to specify directory containing
     * log files generated during test run.
     * @see #logDir
     */
    private static final String PARAM_NAME_LOG_DIR_PREFIX = "-log-dir=";

    /**
     * Prefix for a command line parameter used to specify directory containing
     * HTML templates.
     * @see #templateDir
     */
    private static final String PARAM_NAME_TEMPLATE_DIR_PREFIX = "-template-dir=";

    /**
     * Actual value of -template-dir command line parameter.
     */
    private String templateDir = null;

    /**
     * Actual value of -log-dir command line parameter.
     */
    private String logDir = null;

    /**
     * Actual value of -report-dir command line parameter.
     */
    private String reportDir = null;

    /**
     * Actual value of -date command line parameter.
     */
    private String date = null;

    /**
     * Actual value of -tests command line parameter.
     */
    private Set<String> tests = new TreeSet<String>();

    /**
     * Constructor which starts processing all command line parameters.
     * 
     * @param args
     *            command line parameters stored as an array of Strings
     *            (taken from main() method)
     */
    public CommandLineParameters(String[] args)
    {
        processAllArgs(args);
        checkAllArgs();
    }

    /**
     * Process all command line parameters.
     * 
     * @param args
     *            command line parameters stored as a array of Strings
     *            (taken from main() method)
     */
    private void processAllArgs(String[] args) {
        for (String arg : args)
        {
            // process -template-dir command line parameter
            if (arg.startsWith(PARAM_NAME_TEMPLATE_DIR_PREFIX))
            {
                this.templateDir = arg.substring(PARAM_NAME_TEMPLATE_DIR_PREFIX.length());
            }
            // process -log-dir command line parameter
            else if (arg.startsWith(PARAM_NAME_LOG_DIR_PREFIX))
            {
                this.logDir = arg.substring(PARAM_NAME_LOG_DIR_PREFIX.length());
            }
            // process -report-dir command line parameter
            else if (arg.startsWith(PARAM_NAME_REPORT_DIR_PREFIX))
            {
                this.reportDir = arg.substring(PARAM_NAME_REPORT_DIR_PREFIX.length());
            }
            // process -date command line parameter
            else if (arg.startsWith(PARAM_NAME_DATE_PREFIX))
            {
                this.date = arg.substring(PARAM_NAME_DATE_PREFIX.length());
            }
            // process -tests command line parameter
            else if (arg.startsWith(PARAM_NAME_TEST_PREFIX))
            {
                String testString = arg.substring(PARAM_NAME_TEST_PREFIX.length());
                // test names should be separated by one space
                String[] testArray = testString.split(" ");
                this.tests.addAll(Arrays.asList(testArray));
            }
        }
    }

    /**
     * Basic check if all command line parameters are set and are correct.
     */
    private void checkAllArgs()
    {
        checkTemplateDir();
        checkLogDir();
        checkReportDir();
        checkDate();
        checkTestSet();
    }

    /**
     * Check if -template-dir command line parameter is used.
     * @see #templateDir
     */
    private void checkTemplateDir()
    {
        // check if parameter was specified
        if (this.templateDir == null)
        {
            throw new RuntimeException("template dir is not set");
        }
        // check if directory exists at all
        if (!new File(this.templateDir).isDirectory())
        {
            throw new RuntimeException("template dir is not properly set");
        }
    }

    /**
     * Check if -log-dir command line parameter is used.
     * @see #logDir
     */
    private void checkLogDir()
    {
        // check if parameter was specified
        if (this.logDir == null)
        {
            throw new RuntimeException("log dir is not set");
        }
        // check if directory exists at all
        if (!new File(this.logDir).isDirectory())
        {
            throw new RuntimeException("template dir is not properly set");
        }
    }

    /**
     * Check if -report-dir command line parameter is used.
     * @see #reportDir
     */
    private void checkReportDir()
    {
        // check if parameter was specified
        if (this.reportDir == null)
        {
            throw new RuntimeException("report dir is not set");
        }
        // check if directory exists at all
        if (!new File(this.reportDir).isDirectory())
        {
            throw new RuntimeException("template dir is not properly set");
        }
    }

    /**
     * Check if -date command line parameter is used.
     * @see #date
     */
    private void checkDate()
    {
        if (this.date == null)
        {
            throw new RuntimeException("date is not set");
        }
    }

    /**
     * Check if -tests command line parameter is used.
     * @see #tests
     */
    private void checkTestSet()
    {
        if (this.tests.isEmpty())
        {
            throw new RuntimeException("test list are not set");
        }
    }

    /**
     * Return actual value of logDir attribute.
     *
     * @return actual value of logDir attribute.
     */
    public String getLogDir()
    {
        return this.logDir;
    }

    /**
     * Return actual value of date attribute.
     *
     * @return actual value of date attribute.
     */
    public String getDate()
    {
        return this.date;
    }

    /**
     * Return actual value of templateDir attribute.
     * 
     * @return actual value of templateDir attribute.
     */
    public String getTemplateDir()
    {
        return this.templateDir;
    }

    /**
     * Return actual value of reportDir attribute.
     * 
     * @return actual value of reportDir attribute.
     */
    public String getReportDir()
    {
        return this.reportDir;
    }

    /**
     * Return actual value of tests attribute.
     * 
     * @return actual value of tests attribute.
     */
    public Set<String> getTests()
    {
        return this.tests;
    }

}
