/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.reporter;



import java.util.Locale;



/**
 * Some useful static methods for string handling.
 * 
 * @author Pavel Tisnovsky
 */
public class StringUtils
{

    /**
     * Default locale used for character handling. This field is used for upper case
     * to lower case and also for lower case to upper case conversions
     */
    private static final Locale DEFAULT_LOCALE = Locale.ENGLISH;

    /**
     * Get substring from given string in case that both delimiters are strings
     * too (ie. not null). If the first or the second delimiter are set to null,
     * 0 is used instead of the first delimiter and string.length() instead of
     * the second delimiter. If delimiters can't be found in the given string,
     * null is returned instead of substring.
     * 
     * @param string
     *            given input string
     * @param startDelimiter
     *            first delimiter, might be null
     * @param stopDelimiter
     *            second delimiter, might be null
     * @return substring a new string which is a substring of given input string
     */
    public static String getSubstring(String string, String startDelimiter, String stopDelimiter)
    {
        // silently avoid null pointer exception
        if (string == null)
        {
            return null;
        }

        // compute start and stop delimiters
        int startDelimiterIndex = startDelimiter == null ? 0 : string.indexOf(startDelimiter);
        int stopDelimiterIndex = stopDelimiter == null ? string.length() : string.indexOf(stopDelimiter);
        int startDelimiterLength = startDelimiter == null ? 0 : startDelimiter.length();

        // when delimiters are not found, simply return null string
        if (startDelimiterIndex == -1)
        {
            return null;
        }
        if (stopDelimiterIndex == -1)
        {
            return null;
        }

        // get substring using computed delimiters
        String s = string.substring(startDelimiterIndex + startDelimiterLength, stopDelimiterIndex);

        // trim unnecessary leading and trailing whitespace characters
        return s.trim();
    }

    /**
     * Convert UPPER_CASE_STYLE name into a CamelCase. Example:
     * "VM_VERSION" is converted into "vmVersion"
     * 
     * @param string
     *            input string which could contains names divided by underscore
     *            character
     * @return camel case variant of the input string
     */
    public static String toCamelCase(String string)
    {
        // all underscores should me removed from the string and one
        // character following that underscore should be replaced by the
        // upper case variant of this character
        String[] parts = string.split("_");
        StringBuffer camelCaseString = joinWordsUsingCamelcase(parts);
        return camelCaseString.toString();
    }

    /**
     * Join words using CamelCase style
     * 
     * @param parts
     *            array of words
     * @return camel case variant of all the words
     */
    private static StringBuffer joinWordsUsingCamelcase(String[] parts) {
        StringBuffer camelCaseString = new StringBuffer();
        // properly convert all words (which were separated by underscore)
        // and create one string from them
        for (String part : parts)
        {
            camelCaseString.append(convertToProperCase(part));
        }
        return camelCaseString;
    }

    /**
     * Convert a string to a new form where the first letter is upper case and
     * other letters are lower case
     * 
     * @param string
     *            input string
     * @return input string with capitalized first letter
     */
    public static String convertToProperCase(String string)
    {
        // only first letter should be in upper case,
        // all other letters should be lower case
        return string.substring(0, 1).toUpperCase(DEFAULT_LOCALE) + string.substring(1).toLowerCase(DEFAULT_LOCALE);
    }

    /**
     * Convert "PLACEHOLDER_NAME" into the name of the getter:
     * "getPlaceholderName".
     * 
     * @param placeholderName
     *            name of the placeholder in a form "PLACEHOLDER_NAME"
     * @return converted string name of the corresponding getter in a form
     *         "getPlaceholderName"
     */
    public static String getPlaceholderMethodName(String placeholderName)
    {
        String temporaryName = placeholderName.substring(1);
        temporaryName = temporaryName.substring(1, temporaryName.length() - 1);
        // "get" is common prefix for all getters, word after it should begin with upper case letter
        temporaryName = "get" + StringUtils.toCamelCase(temporaryName.toLowerCase(DEFAULT_LOCALE));
        return temporaryName;
    }

    /**
     * Replace two consecutive spaces by "&nbsp;" (non breaking spaces) to
     * preserve proper formatting of HTML report pages.
     * 
     * @param string
     *            given input string (usually read from .log file)
     * @return string with the given characters replaced.
     */
    public static String replaceSpacesWithNbsps(String string) {
        return string.replaceAll("  ", "&nbsp;&nbsp;&nbsp;");
    }

}
